# router/ReviewRater.py
from collections import defaultdict
from typing import Dict, List, Optional

import numpy as np
from transformers import pipeline


class ReviewRater:
    """Class for generating feature-specific ratings based on sentiment analysis"""

    def __init__(self):
        self.sentiment_analyzer = pipeline("sentiment-analysis")

    def generate_feature_ratings(self, reviews_by_feature: List[Dict[str, str]]) -> Dict[
        str, Dict[str, Optional[float]]]:
        """
        Generate ratings and count sentiment occurrences for each feature.

        Args:
            reviews_by_feature (List[Dict[str, str]]): List of feature-specific reviews with sentiment.

        Returns:
            Dict[str, Dict[str, Optional[float]]]: Feature-wise ratings and sentiment counts.
        """
        feature_data = defaultdict(lambda: {"ratings": [], "positive_count": 0, "negative_count": 0})

        for review_data in reviews_by_feature:
            print("Processing Review:", review_data)  # Debugging line

            feature = review_data.get("feature")
            sentiment = review_data.get("sentiment")

            if not feature or not sentiment:
                print("Skipping due to missing feature or sentiment")
                continue  # Skip invalid entries

            if "positive" in sentiment:
                score = 5
                feature_data[feature]["positive_count"] += 1
            elif "negative" in sentiment:
                score = 1
                feature_data[feature]["negative_count"] += 1
            else:
                score = 3  # Neutral sentiment
                feature_data[feature]["neutral_count"] = feature_data[feature].get("neutral_count", 0) + 1

            feature_data[feature]["ratings"].append(score)

        # Compute average ratings
        for feature, data in feature_data.items():
            data["rating"] = round(np.mean(data["ratings"]), 1) if data["ratings"] else None
            del data["ratings"]  # Remove temporary list after computing rating

        print("Final Feature Data:", dict(feature_data))  # Debugging line
        return feature_data
